<input
	form="doreah_config_form"
	type="number"
	{# min="{{ type.min }}"	max="{{ type.max }}" #}
	step="1"
	placeholder="{{ default }}"
	{% if user %}value="{{ user }}"{% endif %}
	onchange="if (this.value == '') { submitValue('{{ setting }}',null) } else { submitValue('{{ setting }}',Number(this.value)) }" />
